"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _eventemitter = _interopRequireDefault(require("eventemitter3"));

var _util = require("./util");

var _request = require("./request");

var _constrants = require("./constrants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var Promise = require('es6-promise').Promise;

var FetchService = /*#__PURE__*/function () {
  function FetchService() {
    var op = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, FetchService);

    _defineProperty(this, "_eventer", new _eventemitter["default"]());

    _defineProperty(this, "_globalParseResponse", null);

    _defineProperty(this, "_tokenFun", null);

    _defineProperty(this, "_requestNeedToken", false);

    _defineProperty(this, "_requestHeaderTokenName", 'fetch-service-auth-token');

    _defineProperty(this, "_customParseResponse", void 0);

    this._globalParseResponse = op.parseResponse;
    this._tokenFun = op.tokenFun;

    if (op.customParseResponse) {
      this._customParseResponse = op.customParseResponse;
    }

    if (op.requestNeedToken) {
      this._requestNeedToken = true;
    }

    if (op.headers) {
      this._globalHeader = op.headers;
    }

    if (op.mode) {
      this._mode = op.mode;
    }

    if (op.credentials) {
      this._credentials = op.credentials;
    }

    if (op.tokenHeaderName) {
      this._requestHeaderTokenName = op.tokenHeaderName;
    }
  }

  _createClass(FetchService, [{
    key: "__parseResponse",
    value: function __parseResponse(apiOption, data) {
      var resolveData = data;

      if ((0, _util.isFunction)(this._globalParseResponse)) {
        resolveData = this._globalParseResponse(resolveData, apiOption);
      }

      if ((0, _util.isFunction)(apiOption.parseResponse)) {
        resolveData = apiOption.parseResponse(resolveData);
      }

      return resolveData;
    }
  }, {
    key: "__parseParam",
    value: function __parseParam(p, apiOption) {
      if ((0, _util.isFunction)(apiOption.parseParam)) {
        return apiOption.parseParam(p);
      }

      return p;
    }
  }, {
    key: "__parseOption",
    value: function __parseOption(apiOption, param) {
      var _this = this;

      var _apiOption$headers = apiOption.headers,
          headers = _apiOption$headers === void 0 ? {} : _apiOption$headers,
          _apiOption$method = apiOption.method,
          method = _apiOption$method === void 0 ? _constrants.REQUEST_METHOD_GET : _apiOption$method,
          _apiOption$mode = apiOption.mode,
          mode = _apiOption$mode === void 0 ? this._mode : _apiOption$mode,
          _apiOption$credential = apiOption.credentials,
          credentials = _apiOption$credential === void 0 ? this._credentials : _apiOption$credential;
      var _globalHeaders = this._globalHeader;
      var _localHeaders = headers;

      if ((0, _util.isFunction)(this._globalHeader)) {
        _globalHeaders = this._globalHeader(param, apiOption);
      }

      if ((0, _util.isFunction)(headers)) {
        _localHeaders = headers(param, apiOption);
      }

      var newHeaders = (0, _util.deepClone)(Object.assign({}, _globalHeaders, _localHeaders));
      return function (p) {
        _this._eventer.emit(_constrants.EVENT_PARSE_OPTION, apiOption, p, newHeaders);

        var ret = {
          body: p,
          headers: newHeaders,
          method: method
        };

        if (mode) {
          ret.mode = mode;
        }

        if (credentials) {
          ret.credentials = credentials;
        }

        return ret;
      };
    }
  }, {
    key: "__request",
    value: function __request(apiOption, p) {
      var _this2 = this;

      return function (op) {
        _this2._eventer.emit(_constrants.EVENT_WILL_REQUEST, apiOption, op, p);

        var url = (0, _util.renderTemplate)(apiOption.url, p);

        if (apiOption.debug && apiOption.mock) {
          // debug模式下， 使用mock数据模拟数据
          var resolveData = apiOption.mock;

          if ((0, _util.isFunction)(resolveData)) {
            resolveData = resolveData(p);
          }

          return (0, _util.timeoutPromise)(function () {
            return resolveData;
          }, 1000);
        }

        var useOriginResponse = apiOption.useOriginResponse || !!_this2._customParseResponse;
        return (0, _request.request)(url, op, apiOption.useQueue, apiOption.cache, useOriginResponse).then(function (res) {
          if (_this2._customParseResponse) {
            return _this2._customParseResponse(res, url, op, apiOption);
          }

          return res;
        });
      };
    }
  }, {
    key: "__response",
    value: function __response(apiOption) {
      var _this3 = this;

      return function (res) {
        return _this3.__parseResponse(apiOption, res);
      };
    }
  }, {
    key: "__handleError",
    value: function __handleError(apiOption) {
      var _this4 = this;

      return function (err) {
        return !apiOption.noErrorTip && _this4._eventer.emit(_constrants.EVENT_ERROR, err, apiOption);
      };
    }
  }, {
    key: "__ensureToken",
    value: function __ensureToken(apiOption, p) {
      var _this5 = this;

      return function (op) {
        if (_this5._requestNeedToken && apiOption.requestNeedToken !== false && (0, _util.isFunction)(_this5._tokenFun)) {
          return _this5._tokenFun(apiOption, p).then(function (token) {
            if (!op.headers) {
              op.headers = {};
            }

            op.headers[_this5._requestHeaderTokenName] = token;
            return op;
          });
        }

        return op;
      };
    }
  }, {
    key: "parseOption",
    value: function parseOption(apiOption) {
      var _this6 = this;

      return function (p) {
        // 解析传参
        var promise = Promise.resolve(_this6.__parseParam(p, apiOption)) // 解析选项
        .then(_this6.__parseOption(apiOption, p)) // 开始发送请求
        .then(_this6.__ensureToken(apiOption, p)) // 开始发送请求
        .then(_this6.__request(apiOption, p)) // 响应请求
        .then(_this6.__response(apiOption)); // 请求出错

        promise["catch"](_this6.__handleError(apiOption)) // 请求结束
        ["finally"](function () {
          return _this6._eventer.emit(_constrants.EVENT_REQUESTED, apiOption);
        });
        return promise;
      };
    }
  }, {
    key: "on",
    value: function on(eventName, fun) {
      if ((0, _util.isFunction)(fun)) {
        this._eventer.on(eventName, fun);
      }
    }
  }, {
    key: "off",
    value: function off(eventName, fun) {
      this._eventer.off(eventName, fun);
    }
  }]);

  return FetchService;
}();

var _default = FetchService;
exports["default"] = _default;