"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.request = request;
exports.clearRequestCache = clearRequestCache;

var _fetch = _interopRequireDefault(require("./fetch"));

var _cryptoJs = _interopRequireDefault(require("crypto-js"));

var _constrants = require("./constrants");

var _util = require("./util");

var _excluded = ["headers"];

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

// 获取请求选项
function mergeOption(options) {
  if (_typeof(options) !== 'object') {
    options = {};
  }

  var _options = options,
      headers = _options.headers,
      otherOptions = _objectWithoutProperties(_options, _excluded); // 默认选项


  var defaultOption = _objectSpread({
    headers: _objectSpread({}, headers),
    method: _constrants.REQUEST_METHOD_GET
  }, otherOptions);

  var body = defaultOption.body,
      _defaultOption$method = defaultOption.method,
      method = _defaultOption$method === void 0 ? _constrants.REQUEST_METHOD_GET : _defaultOption$method;

  if (typeof defaultOption.method !== 'string' || !_constrants.ALL_METHODS.includes(defaultOption.method.toLowerCase())) {
    defaultOption.method = _constrants.REQUEST_METHOD_GET;
  }

  if (window && window.FormData && body instanceof window.FormData) {
    delete defaultOption.headers['Content-Type'];
  } else if (typeof body === 'number') {
    defaultOption.body = "".concat(body);
  } else if (_typeof(body) === 'object') {
    if (method === _constrants.REQUEST_METHOD_GET || method === _constrants.REQUEST_METHOD_DELETE || defaultOption.headers['Content-Type'] === _constrants.CONTENT_TYPES.FORM_URL) {
      defaultOption.body = (0, _util.json2param)(body);
    } else if (defaultOption.headers['Content-Type'] === _constrants.CONTENT_TYPES.JSON) {
      defaultOption.body = JSON.stringify(body);
    }
  }

  return defaultOption;
} // 处理url


function dealUrl(url, options) {
  var _options$method = options.method,
      method = _options$method === void 0 ? _constrants.REQUEST_METHOD_GET : _options$method,
      body = options.body;

  if ((method === _constrants.REQUEST_METHOD_GET || method === _constrants.REQUEST_METHOD_DELETE) && typeof body === 'string') {
    if (url.indexOf('?') < 0) {
      url += '?';
    } else if (url[url.length - 1] !== '&') {
      url += '&';
    }

    url += body;
    delete options.body;
  }

  return url;
} // 处理成功状态


function dealStatus(useOriginResponse) {
  return function (response) {
    if (useOriginResponse) {
      return Promise.resolve(response);
    }

    if (response.status === 200) {
      return response.text().then(function (txt) {
        return (0, _util.parseJson)(txt);
      });
    } else {
      return Promise.reject(response);
    }
  };
} // 缓存结果


function cacheResponse(md5, useCache, useOriginResponse) {
  return function (json) {
    if (useCache && !useOriginResponse) {
      _requestCache[md5] = json;
    }

    return json;
  };
}

var _fetchQueue = [];
var _fetchQueueStarting = false;

function _addFetchQueue(url, option, useCache) {
  return new Promise(function (resolve, reject) {
    _fetchQueue.push({
      url: url,
      option: option,
      successFun: resolve,
      failFun: reject,
      useCache: useCache
    });

    _startQueue();
  });
}

function _startQueue() {
  if (_fetchQueueStarting) {
    return;
  }

  _fetchQueueStarting = true;
  (0, _util.promiseAll)(_fetchQueue, function (_ref) {
    var url = _ref.url,
        option = _ref.option,
        successFun = _ref.successFun,
        failFun = _ref.failFun,
        useCache = _ref.useCache;

    if (useCache && _requestCache[url]) {
      return Promise.resolve(_requestCache[url]);
    }

    return (0, _fetch["default"])(url, option).then(successFun)["catch"](failFun);
  })["finally"](function () {
    _fetchQueueStarting = false;
  });
}

var _requestCache = {};
/**
 * 请求接口
 * @param url 请求地址
 * @param options fetch Options {method, mode, headers, body, credentials, synchronous}
 * @param useQueue 是否使用请求队列
 * @param useCache 是否缓存结果
 * @param useOriginResponse 是否返回原始response
 * @returns {PromiseLike<T> | Promise<T>}
 */

function request(url, options) {
  var useQueue = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  var useCache = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
  var useOriginResponse = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;
  var lastOption = mergeOption(options);
  var lastUrl = dealUrl(url, lastOption);
  var sha256Str = "".concat(lastUrl, "_").concat(lastOption && lastOption.body && JSON.stringify(lastOption.body) || '');

  var cacheSha256 = _cryptoJs["default"].SHA256(sha256Str).toString(_cryptoJs["default"].enc.Base64);

  if (useQueue) {
    return _addFetchQueue(lastUrl, lastOption, useCache).then(dealStatus(useOriginResponse)).then(cacheResponse(cacheSha256, useCache, useOriginResponse));
  } else {
    if (useCache && _requestCache[cacheSha256]) {
      return Promise.resolve(_requestCache[cacheSha256]);
    }

    return (0, _fetch["default"])(lastUrl, lastOption).then(dealStatus(useOriginResponse)).then(cacheResponse(cacheSha256, useCache, useOriginResponse));
  }
}

function clearRequestCache() {
  _requestCache = {};
}